﻿/*
Название: SelectAndChangeContentType.jsx
Приложение для использования: Adobe InDesign CS3, CS4, CS5, CS6
Версия: 1.0
Язык реализации (Среда): JavaScript (ExtendScript Toolkit 2)
Операционные системы (Платформы): PC, Macintosh (Windows, Mac OS)
Условия распространения: Бесплатно; На Ваш риск
Назначение: Изменяет тип содержимого выделенных объектов (фреймов)
Может использоваться как дополнение для скриптов: MakeImposition.jsx, ImposeMultiPage.jsx

ФУНКЦИОНАЛЬНЫЕ ОГРАНИЧЕНИЯ: 
-------------------------------------------------------
Не обрабатывает: 
- объекты внутри фреймов (или сложных объектов), выделенные инструментом Белая стрелка.
- текстовые фреймы, входящие в текстовые цепочки, содержащие более 1 фрейма.

Техническая поддержка: Sergey-Anosov@yandex.ru
https://sites.google.com/site/dtpscripting

================================================================================

Name: SelectAndChangeContentType.jsx
Application to use with: Adobe InDesign CS3, CS4, CS5, CS6
Version: 1.0
Program language (Environment): JavaScript (ExtendScript Toolkit 2)
Operating systems (Platforms): PC, Macintosh (Windows, Mac OS)
Distribution conditions: Freeware; At your own risk
Functions: Changes content type of selected objects (frames)
Can be used as an addition for the scripts: MakeImposition.jsx, ImposeMultiPage.jsx

FUNCTIONAL LIMITATIONS:
---------------------------------------
Does not process:
- objects inside frames (or complex objects), selected with White arrow tool.
- text frames which belong to the stories, containing more than 1 text frame.

Technical support: Sergey-Anosov@yandex.ru
https://sites.google.com/site/dtpscripting
*/
// название скрипта
var the_title = "SelectAndChangeContentType";
// версия скрипта
var the_version = "1.0";
// активный документ
var AD;
// имя активного документа
var AD_NAME = "";
// файловая система
var WFS = false;
// активный разворот
var AS;
// имя активного разворота
var AS_NAME = "";
// список типов содержимого
var CONT_TYPE_LIST = 
[
	"Text",
	"Graphic",
	"Unassigned"
];
// тип содержимого
var CONT_TYPE;
// индекс типа содержимого
var CONT_TYPE_INDEX = 0;
// текст типа содержимого
var CONT_TYPE_TEXT = "";
// объектный стиль
var OBJ_STYLE;
// объектный стиль выбранный в диалоге
var OBJ_STYLE_VAL;
// текст объектного стиля
var OBJ_STYLE_TEXT;
// массив объектных стилей документа
var obj_styles = new Array();
// количество объектных стилей документа
var N_obj_styles = 0;
// массив имен объектных стилей документа для вывода в диалоге
var OBJ_STYLE_LIST = new Array();
// начальное значение для объектных стилей в диалоге
var OBJ_STYLE_INI = 0;
// стиль символа
var CH_STYLE;
// стиль символа выбранный в диалоге
var CH_STYLE_VAL;
// массив стилей символов документа
var ch_styles = new Array();
// количество стилей символов документа
var N_ch_styles = 0;
// массив имен стилей символов документа для вывода в диалоге
var CH_STYLE_LIST = new Array();
// начальное значение для стилей символов в диалоге
var CH_STYLE_INI = 0;
// стиль параграфа
var PAR_STYLE;
// выбранный стиль параграфа
var PAR_STYLE_VAL;
// количество стилей параграфа документа
var N_par_styles = 0;
// массив имен стилей параграфа документа для вывода в диалоге
var PAR_STYLE_LIST = new Array();
// начальное значение для стилей параграфа в диалоге
var PAR_STYLE_INI = 0;
// вертикальное выравнивание
var VERT_ALIGN;
// значение вертикального выравнивания
var VERT_ALIGN_VAL;
// индекс вертикального выравнивания
var VERT_ALIGN_INDEX = 0;
// текст вертикального выравнивания
var VERT_ALIGN_TEXT = "";
// начальный индекс вертикального выравнивания
var VERT_ALIGN_INI = 1;
// список вертикального выравнивания для текста во фрейме
var VERT_ALIGN_LIST = 
[
	"Top",
	"Center",
	"Bottom",
	"Justify"
];
// вводить ли замещающий текст
var PH = false;
// замещающий текст
var PH_TEXT = "";
// список пригодных для операции объектов
var OBJ_NAME_LIST = 
[
	"TextFrame",
	"Rectangle",
	"Oval",
	"Polygon"	
];
// массив объектов для операции
var OBJECTS = new Array();
// число объектов для операции
var N_OBJ = 0;
// массив выделенного в документе
var DOC_SEL = new Array();
// число выделенных объектов
var N_SEL = 0;
// количество обработанных фреймов
var N_PROC = 0;
// количество ошибок выполнения
var N_ER = 0;
// есть ли заблокированные объекты для обработки
var OBJ_LOCK = false;
// обрабатывать ли заблокированные объекты
var PROC_LOCK = false;
// есть ли открытые пути в объектах для обработки
var OBJ_OPEN_PATH = false;
// обрабатывать ли объекты c открытыми путями
var PROC_OPEN_PATH = false;
// текст для неактивного состояния
var NA_TEXT = "N/A";
// переполнение в текстовых фреймах после операции
var OV = false;
//
// вызов основного модуля
main();
//
// основной модуль
function main()
{
	// вызов подпрограммы начальной проверки
	CHECK();
	// вызов подпрограммы начального диалога
	DIALOG_INI();
};// end main
//
// подпрограмма начальной проверки 
function CHECK()
{
	// проверка наличия открытых документов
	if(app.documents.length == 0 )
	{
		the_exit("No documents are open!");
	};// if
	// определяем файловую систему
	if(File.fs == "Windows") WFS = true; 
	// активный документ
	AD = app.activeDocument;
	// проверяем выделение
	DOC_SEL = AD.selection;
	// количество выделенных объектов
	N_SEL = DOC_SEL.length;
	// если нет выделения вообще
	if( N_SEL == 0 ) the_exit("No selected objects!");
	// проверяем если использована белая стрелка
	if( WHITE_ARROW_USED( DOC_SEL ) )
	{
		var WHITE_ARROW_TEXT = 
		"Warning!\n\n" +
		"Likely White arrow tool to select parts \n"+
		"of complex objects was used!\n\n"+
		"Current version "+ the_version +  " of the script \n"+
		"in some cases can not process that :(";
		the_exit(WHITE_ARROW_TEXT);
	};// if
	// получаем объекты для обработки из выделения
	GET_OBJECTS (DOC_SEL, OBJECTS );
	// количество объектов для операции
	N_OBJ = OBJECTS.length;
	// если не получено объектов для обработки из выделения
	if(  N_OBJ == 0 ) the_exit("No appropriate objects were selected!");
	// проверка текстовых фреймов на цепочки
	if( !STORY_OK( OBJECTS ) )
	{
		var BAD_STORY_TEXT = 
		"Warning!\n\n" +
		"Some text frames to be processed" + "\n" +
		"belong to stories containing\n" +
		"more than 1 text frame!\n\n" +
		"Current version "+ the_version +  " of the script \n"+
		"can not process that :(";
		the_exit( BAD_STORY_TEXT );
	};// if
	// проверяем есть ли заблокированные объекты 
	// и открытые пути в массиве для обработки
	CHECK_LOCK_AND_OPEN_PATHS( OBJECTS );
	// имя активного документа
	AD_NAME = unescape( AD.name );
	// активный разворот
	AS = app.activeWindow.activeSpread; 
	// проверяем версию индизайна
	var the_app_version = parseInt( app.version );
	// если версия индизайна новее CS4
	if( the_app_version >= 7 ) VER_CS5 = true;
	// получаем имя активного разворота
	AS_NAME = GET_SPR_NAME( AS );
	// добавляем к имени активного разворота имена страниц
	var AS_name = AS_NAME + "  " + GET_PAGES_NAME( AS ) + " ";
	// конечное имя активного разворота для диалога
	AS_NAME = AS_name;
	//
	// объектные стили
	// формируем массив имен объектных стилей
	obj_styles = AD.objectStyles;
	N_obj_styles = obj_styles.length;
	for( var i = 0; i < N_obj_styles; i++ )
	{
		OBJ_STYLE_LIST[i] = obj_styles[i].name;
	};// for
	// начальное значение списка имен объектных стилей
	OBJ_STYLE_INI = GET_INI_NAME_INDEX( OBJ_STYLE_LIST );
	//
	// стили параграфа
	// формируем массив имен стилей параграфа
	par_styles = AD.paragraphStyles;
	N_par_styles = par_styles.length;
	for( var i = 0; i < N_par_styles; i++ )
	{
		PAR_STYLE_LIST[i] = par_styles[i].name;
	};// for
	// начальное значение списка имен стилей пераграфа
	PAR_STYLE_INI = GET_INI_NAME_INDEX( PAR_STYLE_LIST );
	//
	// стили символа
	// формируем массив имен стилей символов
	ch_styles = AD.characterStyles;
	N_ch_styles = ch_styles.length;
	for( var i = 0; i < N_ch_styles; i++ )
	{
		CH_STYLE_LIST[i] = ch_styles[i].name;
	};// for
	// начальное значение списка имен стилей символов
	CH_STYLE_INI = GET_INI_NAME_INDEX( CH_STYLE_LIST );
	return;
};//  end CHECK
//
// подпрограмма проверки есть ли заблокированные объекты
// и открытые пути в массиве для обработки
function CHECK_LOCK_AND_OPEN_PATHS( arr )
{
	// цикл по объектам
	for( var i=0; i < arr.length; i++ )
	{
		// текущий объект
		var arr_i = arr[i];
		// проверка заблокированных объектов
		try
		{
			// если объект заблокирован
			if( arr_i.locked ) 
			{
				// в объектах для обработки есть заблокированные
				OBJ_LOCK = true;
			};// if
		} catch ( error ) {};
		// проверка наличия открытых путей в объектах
		try
		{
			if( HAS_OPEN_PATHS( arr_i ) ) 
			{
				OBJ_OPEN_PATH = true;
			};// if
		} catch ( error ) {};
	};// for
	return false;
};// end CHECK_LOCK_AND_OPEN_PATHS
//
// подпрограмма проверки наличия открытых путей в объекте
function HAS_OPEN_PATHS( the_obj )
{
	try
	{
		// цикл по путям объекта
		for( var j = 0; j < the_obj.paths.length; j++ )
		{
			// если путь объекта открытый
			if( the_obj.paths[j].pathType == PathType.OPEN_PATH )
			{
				// в объекте есть открытые пути
				return true;
			};// if
		};// for
	} catch ( error ) {};
	return false;
};// end HAS_OPEN_PATHS
//
// подпрограмма проверки текстовых фреймов для операции
// на принадлежность к цепочкам с более чем 1 фреймом
function STORY_OK( arr )
{
	for( var i = 0; i < arr.length; i++ )
	{
		// текущий объект
		var the_obj = arr[i];
		// если фрейм не текстовый пропускаем его
		if( the_obj.constructor.name != "TextFrame" ) continue;
		// если фрейм текстовый проверяем
		if( !CHECK_STORY( the_obj ) ) return false;
	};// for
	return true;
};// end STORY_OK
//
// подпрограмма определения была ли испрользована белая стрелка
function WHITE_ARROW_USED( arr )
{
	for ( var i = 0; i < arr.length; i++ )
	{
		// родительский объект для текущего объекта
		var parent = arr[i].parent;
		// если родительский объект это фрейм
		// значит была использована белая стрелка
		if( NAME_OK( parent ) ) return true;
	};// for
	return false;
};// end WHITE_ARROW_USED
//
// подпрограмма проверки имени объекта
function NAME_OK( the_obj )
{
	// имя объекта
	var the_name = the_obj.constructor.name;
	// цикл по списку объектов которые могут быть фреймами
	for( var i = 0; i < OBJ_NAME_LIST.length; i++ )
	{
		// если имя объекта совпало с именем из списка
		if( the_name == OBJ_NAME_LIST[i] )
		{
			// значит объект является фреймом
			return true;
		};// if
	};// for
	// если объект графическая линия
	if( the_name == "GraphicLine")
	{
		// но с замкнутым путем (контуром)
		if( !HAS_OPEN_PATHS( the_obj ) )
		{
			// значит объект является фреймом
			return true;
		};// if
	};// if	
	return false;
};// end NAME_OK
//
// подпрограмма поиска текстового фрейма в массиве объектов
function GET_OBJECTS ( arr_in, arr_out )
// arr_in - входящий массив объектов
// arr_out - выходящий массив текстовых фреймов
{
	// подпрограмма проверки находится ли объект уже в выходном массиве
	// obj - проверяемый объект
	// the_arr - массив, принадлежность к которому проверяется
	function IS_IN_ARR( obj, the_arr )
	{
		// ID проверяемого объекта
		var the_obj_id = obj.id;
		// цикл по объектам массива
		for( var i = 0; i < the_arr.length; i++ )
		{
			// если совпали ID значит объект уже в массиве
			if( the_obj_id == the_arr[i].id ) return true;
		};// for
		// если не было совпадений ID значит объект не в массиве
		return false;
	};// end IS_IN_ARR
	//
	// цикл по объектам входного массива
	for( var i =0; i < arr_in.length; i++ )
	{
		// текущий объект
		var the_obj = arr_in[i];
		// родитель текущего объекта
		var the_obj_parent = the_obj.parent;
		// если объект внутри фрейма пропускаем этот объект
		if( NAME_OK( the_obj_parent ) ) continue;
		// проверка по наличию границ
		// если не удалось получить границы пропускаем этот объект
		if( !GBOUNDS( the_obj )[0] ) continue;
		// проверка по имени если объект может быть фреймом
		if( NAME_OK( the_obj ) )
		{
			// если объект еще не в выходном массиве объектов
			if( !IS_IN_ARR( the_obj, arr_out ) )
			{
				// заносим объект в выходной массив объектов
				arr_out.push( the_obj );
				continue;
			};// if
		}
		// если имя не соответствует фрейму
		else
		{
			// поиск по подобъектам внутри объекта
			// массив всех подобъектов
			var sub_objects = new Array();
			//
			// подпрограмма добавления подобъектов определенных типов
			// в общий массив подобъектов
			function GET_SUB_OJECTS( arr )
			{
				try
				{
					for( var j=0; j < arr.length; j++ ) 
					{
						// добавление в общий массив подобъектов
						sub_objects.push( arr[j] );
					};// for
				} catch ( error ) {};// try-catch
				return;
			};// end GET_SUB_OJECTS
			//
			// добавляем прямоугольники
			GET_SUB_OJECTS( the_obj.rectangles );
			// добавляем овалы
			GET_SUB_OJECTS( the_obj.ovals );
			// добавляем текстовые фреймы
			GET_SUB_OJECTS( the_obj.textFrames );
			// добавляем многоугольники
			GET_SUB_OJECTS( the_obj.polygons );
			// добавляем графические линии
			GET_SUB_OJECTS( the_obj.graphicLines );
			// добавляем группы
			GET_SUB_OJECTS( the_obj.groups );
			// обрабатываем полученный массив подобъектов 
			GET_OBJECTS ( sub_objects, arr_out );
		};// if-else
	};// for
	return;
};// end GET_OBJECTS
//
// подпрограмма получения геометрических границ объекта
function GBOUNDS( the_obj )
{
	// результат операции
	var OK = false;
	// границы
	var b;
	try
	{
		// получаем геометрические границы
		b = the_obj.geometricBounds;
		// если удалось получить без ошибки
		OK = true;
	}
	catch ( error ) {};// try-catch
	// возвращаем результат операции и геометрические границы
	return [OK, b];
};// end GBOUNDS
//
// подпрограмма удаления содержимого из фрейма
function CLEAR_FRAME( FRAME )
{
	// тип содержимого
	var CONTENT = FRAME.contentType;
	// если текстовый тип содержимого 
	if( CONTENT == ContentType.TEXT_TYPE )
	{
		try
		{
			// если в текстовом фрейме присутствуют параграфы
			if( FRAME.paragraphs.length > 0 )
			{
				// удаляются все символы всех параграфов
				FRAME.paragraphs.everyItem().characters.everyItem().remove();
			};// if
		} catch ( error ) {};
	};// if
	//
	// подпрограмма удаления объектов
	function REMOVE( arr )
	{
		try
		{
			for( var i = arr.length -1; i >=0; i-- )
			{
				try
				{
					arr[i].remove();
				} catch ( error ) {};// try-catch
			};// for
		} catch ( error ) {};// try-catch
		return;
	};// end REMOVE
	// если графический или неопределенный тип содержимого
	if( (CONTENT == ContentType.GRAPHIC_TYPE) || (CONTENT == ContentType.UNASSIGNED) )
	{
		// удаляем объекты внутри фрейма
		REMOVE( FRAME.pageItems );
		REMOVE( FRAME.allPageItems );
		REMOVE( FRAME.allGraphics );
	};// if
	//  удаляем текстовые пути снаружи фрейма
	REMOVE( FRAME.textPaths );
	return;
};// end CLEAR_FRAME
//
// подпрограмма присвоения типа содержимого
function SET_CONT_TYPE( FRAME)
{
	try
	{
		FRAME.contentType = CONT_TYPE;
	} catch ( error ) { N_ER++ };
	return;	
};// end SET_CONT_TYPE
//
// подпрограмма получения начального значения для 
// списка имен стилей
function GET_INI_NAME_INDEX( list )
{
	try
	{
		// цикл по именам в списке
		for( var i=0; i < list.length; i++ )
		{
			// текущее имя
			var L = list[i];
			// если это не имя по умолчанию
			// у имен по умолчанию обычно в начале и в конце квадратные скобки
			if( (L[0] != "[") && (L[ L.length-1] != "]" ) )
			{
				// считаем его начальным для диалога
				return i;
			};// if
		};// for i
	} catch ( error ) {};
	// если все стили по умолчанию тогда самое первое имя
	return 0;
};// end GET_INI_NAME_INDEX
//
// подпрограмма деактивации / активации списка
function DROP_ACTIVE_DEACTIVE( the_drop, act ) 
{
	// дополнительный пункт НЕАКТИВНО (N/A)
	var NA;
	// если деактивируем 
	if( !act ) 
	{
		try
		{
			// если он уже деактивирован возврат
			if( !the_drop.enabled ) return;
			// добавляем пункт НЕАКТИВНО
			NA = the_drop.add( 'item', NA_TEXT );
			// ставим выделение на этом пункте
			NA.selected = true;
			// собственно деактивируем
			the_drop.enabled = false;
		} catch ( error ) {};
	} 
	// если активируем
	else 
	{
		try 
		{
			// если он уже активирован возврат
			if( the_drop.enabled ) return;
			// удаляем добавленный пункт НЕАКТИВНО
			the_drop.remove(  the_drop.items[ the_drop.items.length - 1 ]  );
			// ставим выделение на 1 пункте
			the_drop.items[0].selected = true;
			// собственно активируем
			the_drop.enabled = true;
		} catch ( error ) {};
	};// if
	return;
};// end DROP_ACTIVE_DEACTIVE
//
// подпрограмма назначения ширины дропдауна
function DDW( d, w )
{
	d.minimumSize.width = d.maximumSize.width = w;
	return;
};// end DDW
//
// подпрограмма окончания скрипта
function the_exit( t )
{
	// если есть текст сообщения
	if( t != "" )
	{ 
		// выводим это сообщение
		alert( t );
	}
	// если нет текста сообщения
	else
	{
		// выводим отчет
		REPORT();
	};// if-else
	// собственно выход
	exit();
};// end the_exit
//
// подпрограмма сокращения имени
// txt - строка на входе
// N - количество символов в выходной строке
function SHORT_NAME(txt, N) 
{
	if ( txt.length > N ) txt = txt.substr(0, N) + "...";
	return txt;
};// end SHORT_NAME
//
// подпрограмма получения статуса разворота
// мастер или обычный разворот
function SPREAD_IS_MASTER( the_spr )
{
	try
	{
		if( the_spr.constructor.name == "MasterSpread" ) return true;
	} catch (error) {};
	return false
};// SPREAD_IS_MASTER
//
// подпрограмма получения имени или номера активного разворота
function GET_SPR_NAME( the_spr )
{
	var the_name = "";
	// если мастер
	if( SPREAD_IS_MASTER( the_spr ) )
	{
		// даем имя мастер разворота
		the_name = the_spr.name;
	}
	// если обычный разворот даем номер
	else
	{
		the_name = the_spr.index + 1;
		the_name = "#" + the_name + " ";
	};// if-else
	return the_name;
};// end GET_SPR_NAME
//
// подпрограмма получения имен для полос разворота
function GET_PAGES_NAME( the_spr )
{
	var P_NAMES = "";
	var P_TEXT = "Page";
	var N_P = the_spr.pages.length;
	// если страниц больше 1 то учитываем множественное число
	if( N_P > 1 ) P_TEXT = P_TEXT +"s";
	P_TEXT = P_TEXT +": ";
	// цикл по страницам разворота
	for(var i = 0; i < N_P; i++)
	{
		// имя текущей страницы
		var the_name = the_spr.pages[i].name;
		// если только 1 старница на развороте
		if( N_P == 1) 
		{
			P_NAMES = P_NAMES + the_name;
			// сразу выход из цикла чтобы не добавлять дефис
			break;
		};// if
		// для первой страницы
		if( i == 0)
		{
			P_NAMES = P_NAMES + the_name;
			// сразу в начало цикла чтобы не добавлять дефис
			continue;
		};// if
		// добавляем дефис
		P_NAMES = P_NAMES+"-"+the_name;
	};// for
	// собственно строка сформирована
	P_NAMES = P_TEXT + P_NAMES;
	// возвращаем ее из подпрограммы
	return P_NAMES;
};// end GET_PAGES_NAME
//
// подпрограмма создания чекбокса
function IN_CB( where, name, len, ini)
{
	var cb = where.add('checkbox');
	cb.text = name;
	cb.value = ini;
	DDW( cb, len );
	return cb;
};// end IN_CB
//
// подпрограмма создания дропдауна
function IN_DATA_DROP( where, data_arr, data_ini, dd_len )
{
	var the_line = where.add('group');
	// создаем строку
	the_line.orientation = 'row';
	the_line.alignChildren = 'left';
	// создаем дропдаун
	var dr = the_line.add('dropdownlist');
	for( var i = 0; i < data_arr.length; i++ )
	{
		dr.add( 'item', data_arr[i] );
	};// for
	DDW( dr, dd_len );
	dr.selection = data_ini;
	return dr;
};// end IN_DATA_DROP
//
// подпрограмма создания текстовой строки в диалоге
function DTEXT( where, txt )
{
	var the_text = where.add('statictext');
	the_text.text = txt;
	return the_text;	
};// end DTEXT
//
// подпрограмма создания поля текстового ввода
function TEXT_FIELD( where, ini_data, data_len )
{
	// создаем строку для всех объектов
	var the_line = where.add('group');
	the_line.orientation = 'row';
	the_line.alignChildren = 'left';
	//
	var data_field = the_line.add('edittext');
	data_field.text = ini_data;
	DDW( data_field, data_len );
	return data_field;
};// end TEXT_FIELD
//
// подпрограмма создания чекбокса с дропдауном в одной строке
function CB_DROP( where, CB_INI, CB_TEXT, CB_TEXT_LEN, DD_ARR, DD_INI, DD_LEN )
{
	var the_line = where.add('group');
	the_line.orientation = 'row';
	the_line.alignChildren = 'left';
	// создаем чекбокс
	var CB = IN_CB( the_line, CB_TEXT, CB_TEXT_LEN, CB_INI);
	CB.onClick = CB_onClick;
	// создаем дропдаун
	var DD = IN_DATA_DROP( the_line, DD_ARR, DD_INI, DD_LEN );
	// подпрограмма реакции на нажатие чекбокса
	function CB_onClick()
	{
		// активируем или декактивируем дропдаун в зависимости от чекбокса
		DROP_ACTIVE_DEACTIVE (DD, CB.value);
		// если включается  чекбокс и активируется дропдаун
		// тогда присваиваем начальное значение дропдауну
		if( CB.value ) DD.selection = DD_INI;
		return;
	};// end CB_onClick
	// возвращаем чекбокс и дропдаун
	return [ CB, DD ];	
};// end CB_DROP
//
// подпрограмма создания начального 
// (главного) диалога
function DIALOG_INI()
{
	var d = new Window('dialog');
	// заголовок диалога
	d.text = the_title + " v." +the_version; 
	d.orientation = 'column';
	d.alignChildren = 'center';
	// шапка
	var d_head = d.add('group');
	d_head.orientation = 'column';
	d_head.alignChildren = 'left';
	// ширина шапки
	DDW( d_head, WM( 295, 340 ) );
	// имя документа
	DTEXT( d_head, "Document: " + SHORT_NAME( AD_NAME, WM( 28, 28) ) );
	// имя активного разворота
	DTEXT( d_head, "Active spread: "+ AS_NAME );
	// объекты в выделении
	DTEXT( d_head, "Selected objects: " + N_SEL.toString() + "   Appropriate objects: " + N_OBJ.toString() );
	// основная панель для опций
	var mp = d.add('panel');
	mp.orientation = 'column';
	mp.alignChildren = 'left';
	// строка для выбора типа содержимого для преобразования
	var type_line = mp.add( 'group' );
	type_line.orientation = 'row';
	type_line.alignChildren = 'left';
	// длина дропдаунов
	var drop_len = WM( 100, 120 );
	// длина текста слева (и чекбоксов)
	var options_text_len = WM( 160, 180 );
	var type_text = type_line.add('statictext');
	type_text.text = "Type of content to convert: ";
	DDW( type_text, options_text_len );
	// выбор типа содержимого
	CONT_TYPE = IN_DATA_DROP( type_line, CONT_TYPE_LIST, 0, drop_len );
	CONT_TYPE.onChange = CONT_TYPE_onChange;
	// если есть заблокированные объекты для обработки 
	if( OBJ_LOCK )
	{
		// выводим запрос на обработку заблокированных объектов
		PROC_LOCK = IN_CB( mp, "Process locked objects", options_text_len, false);
	};// if 
	// если есть объекты с открытыми путями
	if( OBJ_OPEN_PATH )
	{
		// выводим запрос на обработку объектов с открытыми путями
		PROC_OPEN_PATH = IN_CB( mp, "Process objects with open paths", options_text_len + WM( 50, 70), false);
	};// if
	// выбор объектного стиля
	OBJ_STYLE = CB_DROP( mp, false, "Apply object style: ", options_text_len, OBJ_STYLE_LIST, OBJ_STYLE_INI, drop_len );
	DROP_ACTIVE_DEACTIVE( OBJ_STYLE[1], false );
	// выбор стиля символа
	CH_STYLE = CB_DROP( mp, false, "Apply character style: ", options_text_len, CH_STYLE_LIST, CH_STYLE_INI, drop_len );
	DROP_ACTIVE_DEACTIVE( CH_STYLE[1], false );
	// выбор стиля параграфа
	PAR_STYLE = CB_DROP( mp, false, "Apply paragraph style: ", options_text_len, PAR_STYLE_LIST, PAR_STYLE_INI, drop_len );
	DROP_ACTIVE_DEACTIVE( PAR_STYLE[1], false );
	// выбор вертикального выравнивания во фрейме
	VERT_ALIGN = CB_DROP( mp, false, "Apply vertical alignment: ", options_text_len, VERT_ALIGN_LIST, VERT_ALIGN_INI, drop_len );
	DROP_ACTIVE_DEACTIVE( VERT_ALIGN[1], false );
	// строка для замещающего текста
	var placeholder_line = mp.add('group');
	placeholder_line.orientation = 'row';
	placeholder_line.alignChildren = 'left';
	// запрос на ввод замещающего текста
	PH = IN_CB( placeholder_line, "Fill with placeholder text:", options_text_len, false );
	PH.onClick = PH_onClick;
	// начальное значение для замещающего текста
	var PH_TEXT_INI = "AB";
	// поле текстового ввода для замещающего текста
	PH_TEXT = TEXT_FIELD( placeholder_line, NA_TEXT, drop_len );
	PH_TEXT.enabled = false;
	//
	// подпрограмма реакции на чекбокс ввода
	// замещающего текста PH
	function PH_onClick()
	{
		// если включен
		if( PH.value ) 
		{
			// присваиваем начальный замещающий текст
			PH_TEXT.text = PH_TEXT_INI;
			// разрешаем ввод замещающего текста
			if( !PH_TEXT.enabled ) PH_TEXT.enabled = true;
		}
		else
		{
			// замещающий текст N/A
			PH_TEXT.text = NA_TEXT;
			// запрещаем ввод замещающего текста
			if( PH_TEXT.enabled ) PH_TEXT.enabled = false;
		};// if-else
		return;
	};// end PH_onClick
	//
	// подпрограмма активации/деактивации ввода замещающего текста
	function PH_ACTIVE_DEACTIVE( D )
	{
		if( !D )
		{
			if( PH.enabled ) 
			{
				PH.value = PH.enabled = false;
			};
			if( PH_TEXT.enabled )
			{
				PH_TEXT.enabled = false;
			};// if
		}
		else
		{
			if( !PH.enabled ) PH.enabled = true;
		};// if-else
		PH_TEXT.text = NA_TEXT;
		return;
	};// end PH_ACTIVE_DEACTIVE
	//
	// подпрограмма реакции за смену типа контента
	function CONT_TYPE_onChange()
	{
		var N = CONT_TYPE.selection.index;
		//
		// подпрограмма активации/деактивации текстовых стилей
		function TEXT_DATA_ACTIVE_DEACTIVE( F, D )
		{
			if( !D )
			{
				if( F[0].enabled )
				{
					F[0].value = F[0].enabled = false;
				};// if
				if( F[1].enabled )
				{
					DROP_ACTIVE_DEACTIVE( F[1], false );
				};// if
			}
			else
			{
				if( !F[0].enabled ) F[0].enabled = true;
			};// if-else
			return;
		};// end TEXT_DATA_ACTIVE_DEACTIVE
		//
		// подпрограмма активации/деактивации всех текстовых данных
		function ALL_TEXT_DATA_ACTIVE_DEACTIVE( D )
		{
			TEXT_DATA_ACTIVE_DEACTIVE( CH_STYLE, D );
			TEXT_DATA_ACTIVE_DEACTIVE( PAR_STYLE, D );
			TEXT_DATA_ACTIVE_DEACTIVE( VERT_ALIGN, D );
			PH_ACTIVE_DEACTIVE( D );
			return;
		};// end ALL_TEXT_DATA_ACTIVE_DEACTIVE
		//
		// Текстовый тип
		if( N == 0 )
		{
			// активируем все текстовые опции
			ALL_TEXT_DATA_ACTIVE_DEACTIVE( true );
		};// if
		// Графический тип
		if( N == 1 )
		{
			// деактивируем все текстовые опции
			ALL_TEXT_DATA_ACTIVE_DEACTIVE( false );
		};// if
		// Неопределенный тип
		if( N == 2 )
		{
			// деактивируем все текстовые опции
			ALL_TEXT_DATA_ACTIVE_DEACTIVE( false );
		};// if
		return;		
	};// end CONT_TYPE_onChange
	//
	// создание панели кнопок (кнопка ОК, кнопка Cancel)
	var okPanel = d.add('group');
	okPanel.orientation = 'row';
	// кнопка ОК
	var okBtn = okPanel.add('button', undefined, 'OK');
	// кнопка Cancel
	var cancelBtn = okPanel.add('button', undefined, 'Cancel');
	//
	// собственно показываем окно диалога
	var DIALOG_BUTTON = d.show();
	// если выбрана первая кнопка (ОК) выполняем
	if (DIALOG_BUTTON == 1) 
	{
		// тип контента
		CONT_TYPE_INDEX = CONT_TYPE.selection.index;
		CONT_TYPE_TEXT = CONT_TYPE.selection.text;
		if( CONT_TYPE_INDEX == 0 ) CONT_TYPE = ContentType.TEXT_TYPE;
		if( CONT_TYPE_INDEX == 1 ) CONT_TYPE = ContentType.GRAPHIC_TYPE;
		if( CONT_TYPE_INDEX == 2 ) CONT_TYPE = ContentType.UNASSIGNED;
		// обработка открытых путей (если они были :)))
		if( OBJ_OPEN_PATH )
		{
			PROC_OPEN_PATH = PROC_OPEN_PATH.value;
		};// if
		// обработка заблокированных объектов (если они были :)))
		if( OBJ_LOCK )
		{
			PROC_LOCK = PROC_LOCK.value;
		};// if 
		// если текстовый тип контента 
		if( CONT_TYPE_INDEX == 0 )
		{
			// вертикальное выравнивание
			VERT_ALIGN_INDEX = VERT_ALIGN[1].selection.index;
			VERT_ALIGN_TEXT = VERT_ALIGN[1].selection.text;
			VERT_ALIGN = VERT_ALIGN[0].value;
			if( VERT_ALIGN ) 
			{
				if( VERT_ALIGN_INDEX == 0 ) VERT_ALIGN_VAL = VerticalJustification.TOP_ALIGN;
				if( VERT_ALIGN_INDEX == 1 ) VERT_ALIGN_VAL = VerticalJustification.CENTER_ALIGN;
				if( VERT_ALIGN_INDEX == 2 ) VERT_ALIGN_VAL = VerticalJustification.BOTTOM_ALIGN;
				if( VERT_ALIGN_INDEX == 3 ) VERT_ALIGN_VAL = VerticalJustification.JUSTIFY_ALIGN;
			};// if
			// стиль параграфа
			PAR_STYLE_VAL = PAR_STYLE[1].selection.text;
			PAR_STYLE = PAR_STYLE[0].value;
			// стиль символа
			CH_STYLE_VAL = CH_STYLE[1].selection.text;
			CH_STYLE = CH_STYLE[0].value;
			// замещающий текст
			PH = PH.value;
			PH_TEXT = PH_TEXT.text;
		};// if
		// объектный стиль
		OBJ_STYLE_VAL = OBJ_STYLE[1].selection.index;
		OBJ_STYLE_TEXT = OBJ_STYLE[1].selection.text;
		OBJ_STYLE = OBJ_STYLE[0].value;
		//
		// подтверждение выбранных настроек
		if
		( !confirm(
			"Settings confirmation\n\n" + 
			SETTINGS_STR() + "\n" +
			"Warning!\n" +
			"Current content of the objects\n" +
			"to be processed will be removed!\n\n" +
			"Start operation?") 
		) exit();
		//
		// собственно выполнение операции
		OPERATION();
	};// if
	// если в диалоге нажато Cancel
	if (DIALOG_BUTTON == 2) 
	{
		exit();
	};// if
	return;
};// end DIALOG_INI
//
// подпрограмма перевода логического 
// значения в текстовый вид
function YN( x, s )
{
	if( x ) 
	{
		if( s != "" ) return "Yes: " + s
		else return "Yes";
	}
	else return "No";
};// end YN
//
// подпрограмма вывода параметров из главного диалога для подтверждения
function SETTINGS_STR()
{
	var S = "";
	//
	S = S + 
	"Document name: " + AD_NAME + "\n" +
	"Active spread: "+ AS_NAME + "\n" +
	"Selected objects: " + N_SEL.toString() + "\n" +
	"Appropriate objects: " + N_OBJ.toString() + "\n" +
	"Content type to convert: " + CONT_TYPE_TEXT + "\n";
	if( OBJ_LOCK )
	{
		S = S + 
		"Process locked objects: " + YN( PROC_LOCK, "" ) + "\n";
	};// if
	if( OBJ_OPEN_PATH )
	{
		S = S + 
		"Process objects with open paths: " + YN( PROC_OPEN_PATH, "" ) + "\n";
	};// if
	S = S + 
	"Apply object style: " +  YN( OBJ_STYLE, OBJ_STYLE_TEXT ) + "\n";
	if( CONT_TYPE_INDEX == 0 )
	{
		S = S +
		"Apply character style: " + YN(  CH_STYLE,  CH_STYLE_VAL ) + "\n" +
		"Apply paragraph style: " + YN( PAR_STYLE ,  PAR_STYLE_VAL ) + "\n" +
		"Vertical alignment: " + YN( VERT_ALIGN, VERT_ALIGN_TEXT ) + "\n" +
		"Placeholder text: " + YN( PH,  PH_TEXT ) + "\n";
	};// if
	return S;
};// end SETTINGS_STR
//
// подпрограмма получения 
// геометрических границ фрейма
function GBOUNDS( the_obj )
{
	// результат операции
	var OK = false;
	// границы
	var b;
	try
	{
		// получаем геометрические границы
		b = the_obj.geometricBounds;
		// если удалось получить без ошибки
		OK = true;
	}
	catch ( error ) {};// try-catch
	// возвращаем результат операции и геометрические границы
	return [OK, b];
};// end GBOUNDS
//
// подпрограмма разблокирования объекта
function UNLOCK( the_obj )
{
	try
	{
		// разблокируем объект
		the_obj.locked = false;
	}
	// если ошибка считаем что он в заблокированной группе
	catch ( error )
	{
		// родительский объект
		var the_parent = the_obj.parent;
		// если дошли до уровня документа выход
		if( the_parent.constructor.name == "Document" ) return;
		// пытаемся разблокировать родительский объект
		UNLOCK( the_obj.parent );
	};// try-catch
	return;
};// end UNLOCK
//
// подпрограмма выполнения операции
function OPERATION()
{
	// цикл по объектам для обработки
	for( var i = 0; i < N_OBJ; i++ )
	{
		// запоминаем текущий объект как объект документа согласно ID
		// т.к. после изменения содержимого он может стать недоступным
		// для последующих операций 
		var the_obj = AD.pageItems.itemByID( OBJECTS[i].id );
		// если объект имеет открытые пути и не задана обработка открытых путей
		if( HAS_OPEN_PATHS( OBJECTS[i] ) && !PROC_OPEN_PATH )
		{
			// пропускаем этот объект
			continue;
		};// if
		// если объект заблокирован и не задана обработка заблокированных объектов
		if( OBJECTS[i].locked && !PROC_LOCK )
		{
			// пропускаем этот объект
			continue;
		}
		// если задана обработка заблокированных объектов
		else
		{
			// разблокируем
			UNLOCK( OBJECTS[i] );
		};// if-else
		// очистка фрейма
		CLEAR_FRAME( OBJECTS[i] );
		// присвоение типа контента
		SET_CONT_TYPE( OBJECTS[i] );
		// если задано присвоение объектного стиля
		if( OBJ_STYLE )
		{
			try
			{
				the_obj.appliedObjectStyle = obj_styles[ OBJ_STYLE_VAL ];
			} catch ( error ) { N_ER++ };// try-catch
		};// if		
		// если текстовый тип содержимого 
		if( CONT_TYPE_INDEX == 0 )
		{
			try
			{
				// если задано добавление замещающего текста
				if( PH )
				{
					// добавляем текст
					the_obj.contents = PH_TEXT;
				};// if
			} catch ( error ) { N_ER++ };// try-catch
			// если задано выравнивание текста по вертикальному центру
			if( VERT_ALIGN )
			{
				try
				{
					the_obj.textFramePreferences.verticalJustification = VERT_ALIGN_VAL;
				} catch ( error ) { N_ER++ };// try-catch
			};// if
			// если задан стиль параграфа
			if( PAR_STYLE )
			{
				try
				{
					the_obj.paragraphs.everyItem().appliedParagraphStyle = PAR_STYLE_VAL;
				} catch ( error ) { N_ER++ };// try-catch
			};// if
			// если задан стиль символа
			if( CH_STYLE )
			{
				try
				{
					the_obj.paragraphs.everyItem().appliedCharacterStyle = CH_STYLE_VAL;
				} catch ( error ) { N_ER++ };// try-catch
			};// if
			// если был задан замещающий текст
			if( PH )
			{
				// если НЕ был задан стиль параграфа
				if(  !PAR_STYLE )
				{
					try
					{
						// даем выравнивание текста по горизонтальному центру
						the_obj.paragraphs.everyItem().justification = Justification.CENTER_ALIGN;
					} catch ( error ) { N_ER++ };// try-catch
				};// if
				// если НЕ был задан стиль символа 
				// и НЕ был задан стиль параграфа
				if( !CH_STYLE && !PAR_STYLE )
				{
					try
					{
						// присваиваем размер шрифта 100 пунктов
						// как наиболее адекватный для последующих действий :)))
						the_obj.characters.everyItem().pointSize = 100;
						// устраняем переполнение фрейма (если оно было)
						REMOVE_OVERFLOW( the_obj );
					} catch ( error ) { N_ER++ };// try-catch
				};// if
				// если НЕ было задано вертикальное выравнивание
				// и НЕ было задано присвоение объектного стиля
				if( !VERT_ALIGN &&  !OBJ_STYLE  )
				{
					try
					{
						// даем по центру
						the_obj.textFramePreferences.verticalJustification = VerticalJustification.CENTER_ALIGN;
					} catch ( error ) { N_ER++ };// try-catch
				};// if
			};// if
			// проверяем переполнение в текстовом фрейме
			if( !OV )
			{
				OV = CHECK_OVERFLOW( the_obj );
			};// if
		};// if CONT_TYPE_INDEX == 0
		N_PROC++;
	};// for
	// звершение операции
	the_exit("");
};// end OPERATION
//
// подпрограмма определения переполнения в текстовом фрейме
function CHECK_OVERFLOW( the_frame )
{
	var ov = false;
	try
	{
		ov = the_frame.overflows;
	} catch ( error ) {};
	return ov;
};// end CHECK_OVERFLOW
//
// подпрограмма удаления переполнения в текстовом фрейме
function REMOVE_OVERFLOW( the_frame )
{
	try
	{
		// минимальный размер для текста (в пунктах)
		var min_size = 6;
		// максимально количество циклов
		var max_step = 1000;
		// счетчик циклов
		var i = 0;
		do 
		{
			// наличие переполнения во фрейме
			var ov = CHECK_OVERFLOW( the_frame );
			// размер (в пунктах) первого символа первого параграфа
			var s =  the_frame.paragraphs.firstItem().characters.firstItem().pointSize;
			// проверка условий выхода из цикла
			if( !ov || s <= min_size || i >= max_step ) break;
			// уменьшение размера на 1 пункт
			s = s - 1;
			// присваиваем уменьшенный размер всем символам всех параграфов во фрейме
			the_frame.paragraphs.everyItem().characters.everyItem().pointSize = s;
			// увеличение счетчика
			i++;
		} while( true );
	} catch ( error ) {};
	return;
};// end REMOVE_OVERFLOW
//
// подпрограмма завершения работы 
// при наличии цепочек более 1 фрейма
function CHECK_STORY( the_obj )
{
	var story_ok = true;
	try
	{
		// цепочка в которой находится фрейм
		var the_parent_story = the_obj.parentStory;
		// количество фреймов в цепочке, в которой находится фрейм
		var f_num = 0;
		// учитываем разные версии индизайна
		// для новых версий
		if( the_parent_story.hasOwnProperty ("textContainers") )
		{
			f_num = the_parent_story.textContainers.length;
		}
		// для старых версий
		else
		{
			f_num = the_parent_story.textFrames.length;
		};// if-else
		// если количество более 1
		if( f_num > 1 ) story_ok = false;
	} catch (error) {};// try-catch
	return story_ok;
};// end CHECK_STORY
//
// подпрограмма вывода сообщения о результатах работы
function REPORT()
{
	// формирование сообщения
	var S = "Report\n\n" +
	SETTINGS_STR() + "\n" +
	"Processed: " + N_PROC + "\n" +
	"Errors: "+ N_ER + "\n\n";
	if( OV )
	{
		S = S +
		"Warning:" + "\n" +
		"There is overset text" + "\n" +
		"in some text frames!";
	};// if
	// вывод сообщения
	alert( S );
	return;
};// end REPORT
//
// подпрограмма присвоения значения переменной
// в зависимости от файловой системы
function WM( W, M )
{
	// если Windows
	if( WFS ) return W
	// если Macintosh
	else return M;
};// end WM